#include <LPC2103.h>
#include "poten.h"

#define TEMT    (1<<6)
#define LED1    0x2000
#define LED2    0x01000000
#define Button1 0x00800000
#define Button2 0x200
#define Button3 0x400000
#define CR      0x0D
#define LF      0x0A

char f_10ms = 0;
char f_50ms = 0;//Button pause
char f_100ms = 0;
char f_200ms = 0;//Event LED
char f_1000ms = 0;//Even ADC to UART
const char ascii[] = "0123456789ABCDEF";//Convert byte

void send_uart0(unsigned char b)
{
while( !(U0LSR & TEMT) );
U0THR = b;
}

void PrintByte(unsigned char b)
{
send_uart0(ascii[b >> 4]);
send_uart0(ascii[b & 0xF]);
}

void PrintString(const char *s)
{
while(*s)
	{
	if(*s == '\n')
		send_uart0('\r');
	send_uart0(*s);
	s++;
	}
}

__irq void T0_Isr(void)
{
static unsigned char cnt = 0;
static unsigned char cnt2 = 0;
static unsigned char cnt3 = 0;
static unsigned char cnt4 = 0;

f_10ms = 1;//Event ADC

if(++cnt4 > 5)
	{
	cnt4 = 0;
	f_50ms = 1;
	}
if(++cnt > 40)
	{
	cnt = 0;
	f_200ms = 1;
	}
if(++cnt3 > 3)
	{
	cnt3 = 0;
	f_100ms = 1;
	}
if(++cnt2 > 250)
	{
	cnt2 = 0;
	f_1000ms = 1;
	}
T0IR = 1;
VICVectAddr = 0;
}

void delay_ms(long int ms)
{
  long int i,j;
  for(i=0;i<ms;i++)
  for(j=0;j<6666;j++);
}

void initialize(void)
{
  /* initialize the serial interface   */
  PINSEL0 = 0x00000005;           /* Enable RxD0 and TxD0                     */
  U0FCR = 0x7;//Enable FIFO
  U0LCR = 0x83;                   /* 8 bits, no Parity, 1 Stop bit            */
  U0DLL = 18;                     /* 9600 Baud Rate @ 6MHz VPB Clock 98 15MHz VPB (39 24MHz)        */
  U0DLM = 0;
  U0LCR = 0x03;                   /* DLAB = 0*/
  U0FDR = 0xC1;//   DIVADDVAL  MULVAL

//Port LED1 P0.13
  PINSEL0 |= 0x00002000;
  IODIR |= 0x00002000;
  IOCLR |= 0x00002000;

//Port LED2 P0.24
  PINSEL0 |= 0x01000000;
  IODIR |= 0x01000000;
  IOCLR |= 0x01000000;

//Port Button1 P0.23
//  PINSEL1 |= 0x00000000;
//  IODIR |= 0x00000000;
  //IOSET |= 0x00800000;

//Port Button2 P0.09  
//  IOSET |= 0x200;

//Port Button3 P0.22  
//  IOSET |= 0x400000;

//ADC
  PINSEL0 |= 0xF00000;//in ADC AD0.3 and AD0.4
  PINSEL0 |= 0x3000000;//in ADC AD0.5
  PINSEL1 |= 0xC0000;//in ADC AD0.6


//Timer0
  T0MR0 = 20000;
  T0MCR = 3;
  T0TC = 0;
  T0TCR = 1;

  VICVectAddr0 = (unsigned long) T0_Isr;
  VICVectCntl0 = 0x24;
  VICIntEnable |= 0x10;
}

int main (void)
{
unsigned char diode = 0;
unsigned char adc = 0;//ADC temp
char adcstart = 0;//ADC Start
unsigned char pt3 = 0x80;//old ADC data
unsigned char pt4 = 0x80;//old ADC data
unsigned char pt5 = 0x80;//old ADC data
unsigned char pt6 = 0x80;//old ADC data
char bt1 = 1;//old button
char bt2 = 1;//old button
char bt3 = 1;//old button

	initialize();

//privet
//PrintString("Start program.\r");

	while (1)
	{
	if( f_50ms )//Scan Button P0.23
		{
		f_50ms = 0;
		if( ( IOPIN & Button1 ) && !bt1 )
			{
			send_uart0('B');
			send_uart0('1');
			send_uart0('1');
			send_uart0(LF);
			send_uart0(CR);
			bt1 = 1;
			}
		else
			{
			if( !( IOPIN & Button1 ) && bt1 )
				{
				send_uart0('B');
				send_uart0('1');
				send_uart0('0');
				send_uart0(LF);
				send_uart0(CR);
				bt1 = 0;
				}
			}
		if( ( IOPIN & Button2 ) && !bt2 )
			{
			send_uart0('B');
			send_uart0('2');
			send_uart0('1');
			send_uart0(LF);
			send_uart0(CR);
			bt2 = 1;
			}
		else
			{
			if( !( IOPIN & Button2 ) && bt2 )
				{
				send_uart0('B');
				send_uart0('2');
				send_uart0('0');
				send_uart0(LF);
				send_uart0(CR);
				bt2 = 0;
				}
			}
		if( ( IOPIN & Button3 ) && !bt3 )
			{
			send_uart0('B');
			send_uart0('3');
			send_uart0('1');
			send_uart0(LF);
			send_uart0(CR);
			bt3 = 1;
			}
		else
			{
			if( !( IOPIN & Button3 ) && bt3 )
				{
				send_uart0('B');
				send_uart0('3');
				send_uart0('0');
				send_uart0(LF);
				send_uart0(CR);
				bt3 = 0;
				}
			}
		}
	if(f_100ms)//Event ADC
		{
		f_100ms = 0;
		f_1000ms = 0;
		switch( adcstart )
			{
			case 0:
			ADCR = 0x200308;//AIN3
			ADCR |= 0x1000000;//Start ADC
			break;
			case 1:
			adc = ADDR3 >> 8;
			if( ( adc > (pt3+1)) || adc < (pt3-1) )
				{
				pt3 = adc;
				send_uart0( 'm' );
				send_uart0( '1' );
				if( adc >= 0x80 ) PrintByte( table_pot3[ adc ] );
				else PrintByte( 0x80 - table_pot3[ adc ] );
//				PrintByte( table_pot3[ adc ] );
				send_uart0(LF);
				send_uart0( CR );
				}
			break;
			case 2:
			ADCR = 0x200310;//AIN4
			ADCR |= 0x1000000;//Start ADC
			break;
			case 3:
			adc = ADDR4 >> 8;
			if( ( adc > (pt4+1)) || adc < (pt4-1) )
				{
				pt4 = adc;
				send_uart0( 'm' );
				send_uart0( '2' );
				if( adc >= 0x80 ) PrintByte( table_pot4[ adc ] );
				else PrintByte( 0x80 - table_pot4[ adc ] );
				send_uart0(LF);
				send_uart0( CR );
				}
			break;
			case 4:
			ADCR = 0x200320;//AIN5
			ADCR |= 0x1000000;//Start ADC
			break;
			case 5:
			adc = ADDR5 >> 8;
			if( ( adc > (pt5+1)) || adc < (pt5-1) )
				{
				pt5 = adc;
				send_uart0( 'P' );
				send_uart0( '1' );
				if( adc >= 123 ) PrintByte( table_pot1[ adc ] );
				else PrintByte( 0x80 - table_pot1[ adc ] );
				send_uart0( LF );
				send_uart0( CR );
				}
			break;
			case 6:
			ADCR = 0x200340;//AIN6
			ADCR |= 0x1000000;//Start ADC
			break;
			case 7:
			adc = ADDR6 >> 8;
			if( ( adc > (pt6+1)) || adc < (pt6-1) )
				{
				pt6 = adc;
				send_uart0( 'P' );
				send_uart0( '2' );
				if( adc >= 123 ) PrintByte( table_pot2[ adc ] );
				else PrintByte( 0x80 - table_pot2[ adc ] );
				send_uart0( LF );
				send_uart0( CR );
				}
			break;
			}
		if( adcstart >= 7 ) adcstart = 0;
		else adcstart ++;
		}

	if(f_200ms)//Event LED
		{
		f_200ms = 0;
		if(diode)
			{
			diode = 0;
			IOCLR |= LED1;
			IOCLR |= LED2;
			}
		else
			{
			diode = 1;
			IOSET |= LED1;
			IOSET |= LED2;
			}
		}

	if(f_1000ms)//Event Active
		{
		f_1000ms = 0;
		send_uart0('O');
		send_uart0('K');
		send_uart0(LF);
		send_uart0(CR);
		}
	}
}
